###############  SETUP  ##############
TARGET_SEARCH_PATH = 'target/' # i will look for target sf's here - relative to the location of the concateSf.py script
CORPUS_SEARCH_PATH = 'source/'
PLOT_PATH = 'output/'
SOUNDFILE_EXTENSIONS = ['.aiff', '.aif', '.wav', '.au'] # load dir -> skip files without these extensions; not case sensative
RANDOM_SEED = None # None=use timestamp; integer for repeatable pseudo random results
VERBOSITY = 2 # turn off printing altogether with 0.
USE_PROGRESS_BAR = True # use progress printing bars
OUTPUT_FLOAT_PRECISION = 3 # the decimal precision of numbers in the output text files/scores
PRINT_LENGTH = 80
ALERT_ON_ERROR = True # play the system alert sound when exiting with an error.  irritating, but then again you probably deserve it.
PRINT_SELECTION_HISTO = False
PRINT_SIM_SELECTION_HISTO = True
PRINT_ELAPSED_TIMES = True

################  DESCRIPTOR COMPUTATION SETTINGS  ################
DESCRIPTOR_FORCE_ANALYSIS = False # will force recomputation every time
DESCRIPTOR_WIN_SIZE_SEC = 0.04643990929708 # ~2048 @ 44100
DESCRIPTOR_HOP_SIZE_SEC = 0.01160997732427 # overlap, this sets the temporal resolution
# ircamdescriptor
IRCAMDESCRIPTOR_RESAMPLE_RATE = 12500 # 12500 = ircam's default value
IRCAMDESCRIPTOR_WINDOW_TYPE = 'blackman'
IRCAMDESCRIPTOR_NUMB_MFCCS = 13
IRCAMDESCRIPTOR_F0_MAX_ANALYSIS_FREQ = 5000
IRCAMDESCRIPTOR_F0_MIN_FREQUENCY = 20
IRCAMDESCRIPTOR_F0_MAX_FREQUENCY = 5000
IRCAMDESCRIPTOR_F0_AMP_THRESHOLD = 30
IRCAMDESCRIPTOR_F0_QUALITY = 0.1
# supervp
SUPERVP_NUMB_PEAKS = 12 
# clusteranal
CLUSTERANAL_DESCRIPTOR_DIM = ['mfcc1', 'mfcc2', 'mfcc3']
CLUSTERANAL_NUMB_CLUSTS = 8 



##############  OUTPUT FILES  #############
# FOR EACH VARIABLE ENDING IN _FILEPATH:
#    a value of None will turn off the creation of this file
#    a string path will create this file in the location specified
CSOUND_SCORE_FILEPATH = 'output/csoundScore.txt'
CSOUND_RENDER_FILEPATH = 'output/output.aiff'
LOG_FILEPATH = 'output/log.txt'
MIDI_FILEPATH = 'output/midiFile.mid'
TARGET_SEGMENT_LABELS_FILEPATH = 'output/tgtLabels.txt'
SUPERIMPOSITION_LABEL_FILEPATH = 'output/superimpositionLabels.txt'
QLIST_FILEPATH = None #'output/qlistScore.txt'
LISP_FILEPATH = None #'output/events.lisp'
OM_SCORE_FILEPATH = None #'output/openMusicScore.txt'
TGT_OM_SCORE_FILEPATH = None #'output/tgtOmScore.txt'
BACH_SCORE_FILEPATH = None #'output/bachCommands.txt'
SEGMENT_LABELS_FILEPATH = None #'output/outputLabels.txt'
PARAM_SCORE_FILEPATH = None #'output/paramscore.txt'
CSOUND_OUTPUT_MIX_FILEPATH = None #'output/outputMix.aiff'
MAXMSP_DICT_PATH = None #'output/output.json'


CSOUND_WRITE_SCORE_COMMENTS = False
CSOUND_RENDER_WITH_ULIMIT = True
CSOUND_PLAY_RENDERED_FILE = True
CSOUND_RENDER_SEPARATE_FILE_FOR_EACH_CORPUS_ENTRY = False
CSOUND_RENDER_SEPARATE_FILE_FOR_EACH_CORPUS_OVERLAP = False
CSOUND_SCORE_DURATION_TYPE = 'corpusDur' # corpusDur || corpusEffDur || targetDur || simCalcDur
CSOUND_STRETCH_CORPUS_DURATIONS_TO_MATCH_TARGET = 0
CSOUND_OUTPUT_MIX_PAN = 'middle' # 'middle' || 'leftRight'



############ TARGET  ############
TARGET_STRETCH_TIME = 1 # uses ircam supervp
TARGET_STRETCH_FLAGS = '-Afft -Np0 -M0.092879802s -oversamp 8 -Whamming -P1 -td_thresh 1.2 -td_G 2.5 -td_band 0,22050 -td_nument 10 -td_minoff 0.02s -td_mina 9.9999997e-06 -td_minren 0 -td_evstre 1 -td_ampfac 1.5 -td_relax 100 -td_relaxto 1 -FCombineMul -shape 1 -Vuf -4'

TARGET_DESCRIPTOR_MODIFY = [] # happens before normalization!
TARGET_PLOT_METRICS = False
TARGET_ONSET_DESCRIPTORS = {'power-odf-7': 1}
TARGET_ONSET_FORCE_MAX = {}
TARGET_ONSET_THRESHOLD_SIM_PENALITY = +0
TARGET_ONSET_THRESHOLD_ACTIVE_PENALITY = +0
TARGET_SEGMENT_OFFSET_DB_ABS_THRESH = -80.
TARGET_SEGMENT_OFFSET_DB_REL_THRESH = +18

TARGET_ONSET_ENVELOPE = 'raw' # orig=unsubtracted, raw=subtracted
TARGET_OFFSET_ENVELOPE = 'raw' # orig=unsubtracted, raw=subtracted

TARGET_LOAD_SEGMENTS_FILEPATH = None
# impose a grid shape to the target's power
# None = do nothing
# else: a dict:
#  {'period': 0.125, 'duration': 0.1, 'slope': 2, 'norm': True, 'randomnessSec': 0.05}
# takes place before ag.loadAnalsOnFly
TARGET_MAKE_GRID_ATTENUATION = None 

##########  CORPUS TEMPORAL RESTRICTIONS   #######
CORPUS_GLOBAL_ATTRIBUTES = {}
MW_DEFINE_INSTRUMENTS = []
LIMIT_CORPUS_BY_DESCRIPTOR = {}
VOICE_PATTERN = []
CORPUS_RESTRICT_DISCRIPTOR_IN_TIME = []
CORPUS_RESTRICT_NUMBER_STREAMS = 0
MAX_ACTIVE_VOICES = None
ROTATE_VOICES = False


##############  DATA NORMALIZATION  #############
NORMALIZE_METHOD = 'stddev' # not fixed yet/./././/./././/./././/./././/./././
NORMALIZE_TARGET_SEGMENTS_ONLY = True # generally a good idea
NORMALIZE_SCALE_STD_DEV = 1
NORMALIZE_CLUSTER_MAPPING = 'leastSqrError'
NORMALIZE_CLUSTERS_METHOD = 'stddev' # 'stddev' || 'minmax'

#######  CONCATENATE SELECTION  #######
SUPERIMPOSE = si()
ALWAYS_MAKE_COMPLETE_MATCHING_RESULTS = False
VOICE_RESTRICT_PER_SEGMENT = False
RANDOMIZE_AMPLITUDE_FOR_SIM_SELECTION = False



# still have to move over to descriptor attributes
# still have to move over to descriptor attributes
# still have to move over to descriptor attributes
# still have to move over to descriptor attributes
# still have to move over to descriptor attributes
# still have to move over to descriptor attributes
SIM_CALC_PEAK_ALIGN = 'none' # none || align
CONCATENATE_PEAK_ALIGN_ENVELOPE = 'raw' # orig=unsubtracted, raw=subtracted



################  CONCATENATE AMPLITUDE   ################
OUTPUT_GAIN = 0
POST_SELECTION_AMP_SCALE = 'None' # "None" || "lstSqr" || "peakAmp"
POST_SELECTION_AMP_MIN = -12 # only if POST_SELECTION_AMP_SCALE != None
POST_SELECTION_AMP_MAX = +12
POST_SELECTION_AMP_PEAK_AMP_METRIC = 'power-seg'



################  CONCATENATE TIME MANIPULATION  ################
TEMPO_CHANGE = 60
TIME_OFFSET = 0.0
ALIGN_NOTE_PEAKS = False
RHYTHM_QUANTIZE = 0
RHYTHM_QUANTIZE_METHOD = 'aggregate'
BACH_TEMPO = 60
BACH_METER = '4/4'
CONCATENATE_PEAK_ALIGN = 'none'


########  EXPERIMENTAL / HACKY  ########
JSON_DATA_FILEPATH = 'output/audioguide-data.json.txt' # for different types of internally computed analyses
LOAD_ADDITIONAL_DESCRIPTORS = []
SEARCH_CORPUS_PARAM_INTERPOLATE = False # experimental
SEARCH_CORPUS_PARAM_INTERPOLATE_POINTS = 5
ORDER_CORPUS_BY_DESCRIPTOR = []
CORPUS_POCKETING = False
CORPUS_POCKETING_ROTATE = False
CORPUS_POCKET_SEGMENT_RANDOM_RANGE = [1, 4]

MATCH_PEAKS = False # a peak matching algorithm unique for peakFreqs-N
MATCH_PEAKS_AMP_THRESH = 0.1
MATCH_PEAKS_TRANS_UP_AND_DOWN = 6. # in semitones
MATCH_PEAKS_TRANS_ITERATE_INTERVAL = 1 # what interval to search by...
MATCH_PEAKS_CHROMA_ONLY = True
MATCH_PEAKS_USE_PARTIAL_AMPLITUDE = True