import sys, os 
import utilities

class parseOptions:
	def __init__(self, AudioGuidePath, ScriptPath, loadOptionsFromFile, pythonicUserOptions, defaultOptionsPath):
		# audioGuide user options are taken from 2 sources:
		#	from loadOptionsFromFile is it is not None
		#	from pythonicUserOptions
		self.loadOptionsFromFile = loadOptionsFromFile
		self.optionsfileDir = os.path.split(loadOptionsFromFile)[0]
		self.pythonicUserOptions = pythonicUserOptions
		self.userOps = {}
		if loadOptionsFromFile != None:
			self.userOps.update(file2optionsDict(loadOptionsFromFile))
		if type(pythonicUserOptions) == str:
			# load a string into the options
			self.userOps.update(string2optionsDict(pythonicUserOptions))
		elif type(pythonicUserOptions) == dict:
			# load a dict into the options
			self.userOps.update(pythonicUserOptions)
		# load defaults from disk
		versionDefPath = os.path.join(AudioGuidePath,defaultOptionsPath)
		if not os.path.isfile(versionDefPath):
			utilities.error("ops","The default options file "+versionDefPath+" does not exist!")
		self.defaultOps = file2optionsDict(versionDefPath)	
		############################
		## complete all pathnames ##
		############################
		self.abpspaths = {}
		for dicty in [self.userOps, self.defaultOps]:
			for k, v in dicty.iteritems():
				if k.find('FILEPATH') != -1 or k.find('_SEARCH_PATH') != -1:
					if v != None and not os.path.isabs(v):
						dicty[k] = os.path.join(ScriptPath, v)
						self.abpspaths[k] = dicty[k]
		############################################
		## complete target, corpus, package paths ##
		############################################		
		self.userOps['TARGET'].name = testPathDie(self.userOps['TARGET'].name, (self.optionsfileDir, self.TARGET_SEARCH_PATH, ScriptPath))
		for cobj in self.userOps['CORPUS']:
			cobj.name = testPathDie(cobj.name, (self.optionsfileDir, self.CORPUS_SEARCH_PATH, ScriptPath))
			if cobj.segmentationFile != None:
				print cobj.segmentationFile
				cobj.segmentationFile = testPathDie(cobj.segmentationFile, (self.optionsfileDir, self.CORPUS_SEARCH_PATH, ScriptPath))
		##################################
		## write full path options file ##
		##################################
		fh = open('/tmp/ag-fullpath-ops.txt', 'w')
		for dicty in [self.userOps, self.abpspaths]:
			for k, v in dicty.iteritems():
				if type(v) == str:
					fh.write( '%s = "%s"\n'%(k, v) )
				else: fh.write( '%s = %s\n'%(k, v) )
		fh.close()
		if self.userOps.has_key('VERBOSITY'): self.v = self.userOps['VERBOSITY']
		else: self.v = self.defaultOps['VERBOSITY']

		# get options for correct formatting and type
		checkOptionCasting(self.userOps)
		#self.checkAlternativeNames()
		# check options to make sure everything exists
		self.checkOptions()
		self.userOps['AG_PATH'] = AudioGuidePath
		self.userOps['SCRIPT_PATH'] = ScriptPath
		self.userOps['OPS_PATH'] = loadOptionsFromFile
		self.userOps['DEFAULT_PATH'] = versionDefPath
		# these three are kept to that if options are changed we
		# can keep track of what needs to be recomputed.
		self.needToInitalize = True
		self.needToRemakeAnalLists = False
		self.needToLoadTgt = True
		self.needToLoadSrc = True
		self.needToNormalize = True
		self.needToRewindTgt = False
	############################################################################
	def checkOptions(self):
		for item in self.userOps:
			if not self.defaultOps.has_key(item) and item not in ['TARGET', 'CORPUS', 'SEARCH']:
				utilities.error('ops', 'AudioGuide does not have an option called %s!'%item)
	############################################################################
	def __getattr__(self, name): # for ops.OPTION_NAME calls...
		if self.userOps.has_key(name): return self.userOps[name]
		elif self.defaultOps.has_key(name): return self.defaultOps[name]
		else: return self.userOps[name]
		#else: # die
		#	utilities.error("ops", "AudioGuide needs an option called "+name+" but your options file does not have it!")
	############################################################################
	def __setattr__(self, name, val): # so we can keep track of changed options, know what needs to be redone.
		if self.__dict__.has_key(name) and self.__dict__[name] == val: pass # not changed
		self.__dict__[name] = val # normal operation
		if name in ['TARGET', 'TARGET_FADE_IN_SEC', 'TARGET_FADE_OUT_SEC', 'TARGET_ONSET_THRESHOLD', 'TARGET_ONSET_DESCRIPTORS', 'MAX_SIMULTANEOUS_SAMPLES', 'TARGET_ONSET_THRESHOLD_SIM_PENALITY', 'MAX_SEGMENTS_PER_ONSET', 'TARGET_CORRECT_ODF_LATENCY', 'ODF_LATENCY_FRAMES', 'ODF_NUMB_AVG', 'TARGET_SEGMENT_DROP_RATIO', 'TARGET_OFFSET_DB_ABOVE_MIN', 'TARGET_SEGMENT_LENGTH_DROP_METHOD', 'TARGET_ONSET_ENVELOPE', 'TARGET_OFFSET_ENVELOPE', 'TARGET_MIN_SEGMENT_LENGTH', 'TARGET_MAX_SEGMENT_LENGTH', 'TARGET_INCREMENT', 'TARGET_INCREMENT_ENVELOPE', 'TARGET_MIN_INCREMENT', 'TARGET_SCALE_AMP']: self.needToLoadTgt = True ; self.needToNormalize = True
		if name in ['CORPUS', 'LIMIT_CORPUS_BY_DESCRIPTOR', 'CAN_USE_SAME_SEGMENT_TWICE', 'CAN_REUSE_SEGMENT_AFTER', 'CORPUS_RESTRICT_DISCRIPTOR_IN_TIME', 'CORPUS_RESTRICT_NUMBER_STREAMS', 'VOICE_RESTRICT_ENTRANCES_IN_TIME', 'MAX_ACTIVE_VOICES', 'ROTATE_VOICES', 'SOUNDFILE_EXTENSIONS', 'SPECTRAL_POWER_THRESHOLD']: self.needToLoadSrc = True ; self.needToNormalize = True
		if name in ['NORMALIZE_METHOD', 'NORMALIZE_DEFAULT_METHOD', 'NORMALIZE_SCALE_STD_DEV', 'NORMALIZE_TARGET_SEGMENTS_ONLY']: self.needToNormalize = True
		
		if name in ['SEARCH', 'FIRST_PASS_DESCRIPTORS', 'FIRST_PASS_PERCENT']:
			self.needToRemakeAnalLists = True
			self.needToNormalize = True
	
	############################################################################
	def writeConfigurationFile(self, filename):
		neverWrite = []
		orderedOps = readOptionOrder(self.userOps['DEFAULT_PATH'])
		toWrite = []
		nothingWritten = True
		skipWrite = ['AG_PATH', 'SCRIPT_PATH', 'OPS_PATH', 'DEFAULT_PATH']
		
		addUserOps = [] # for extra user ops that are not in defaults..  
		for name, val in self.userOps.iteritems():
			if name not in skipWrite and not self.defaultOps.has_key(name): # items that are in userops but not defaults
				addUserOps.append(name)
		for i, op in enumerate(addUserOps): # add unsupplied user ops to the write list
			orderedOps[0].insert(i+1, op)
		
		for array in orderedOps:
			sectionID = array[0]
			sectionWritten = False
			for name in array[1:]:
				if self.userOps.has_key(name) and (not self.defaultOps.has_key(name) or self.userOps[name] != self.defaultOps[name]):
					if not sectionWritten:
						sectionString = makeSectionName(sectionID)
						if not nothingWritten: sectionString = "\n"+sectionString
						toWrite.append(sectionString)
						sectionWritten = True
						nothingWritten = False
					toWrite.append(name+' = '+self.filterSpecialNames(name))
		configFile = open(filename, 'w')
		for line in toWrite:
			configFile.write(line+'\n')
		configFile.close()
	############################################################################
	def filterSpecialNames(self, name):
		if name in ['DISTANCE_DESCRIPTORS', 'FIRST_PASS_DESCRIPTORS']:
			tmp = {}
			for key, val in self.userOps[name].iteritems():
				if type(val) == tuple:
					if val[0] == 0: continue
				else: 
					if val == 0: continue
				tmp[key] = val
			string = str(tmp)
		else:
			if type(self.userOps[name]) == str:
				string = '"'+str(self.userOps[name])+'"'
			else:
				string = str(self.userOps[name])
			
		return string


def testPathDie(tpath, dirs):
	if os.path.exists(tpath): return os.path.abspath(tpath)
	for d in dirs:
		if os.path.exists(os.path.join(d, tpath)): return os.path.join(d, tpath)
	# shouldn't get here
	utilities.error('file', 'cannot find file "%s"!!!'%tpath)
	sys.exit(1)
	

def makeSectionName(sectionID):
	lenP = (56-len(sectionID))/2
	return "#"*lenP+sectionID+"#"*lenP

	
def readOptionOrder(file):
	fh = open(file, 'r')
	lines = fh.readlines()
	fh.close()
	orderedOps = []
	sectionOps = ['  REQUIRED  '] # first section is REQUIRED
	sectionCounter = 0
	for line in lines:
		line = line.replace('\n','')
		if line == '': continue
		if line[0:3] == '###':
			sectionCounter += 1
			orderedOps.append(sectionOps)
			sectionOps = [line.replace('#','')]
		elif line[0] == '#': continue
		else:
			sectionOps.append(line.split()[0])
	orderedOps.append(sectionOps) # append last section
	return orderedOps



def checkErrandKeywordArgs(defs, keyws):
	for k in keyws:
		if not defs.has_key(k):
			utilities.error('options', 'csf object does not have a keyword argument "%s"'%k)


class tsf:
	def __init__(self, name, **kwargs):
		self.name = name
		self._defaults = {'end': None, 'start': None, 'scaleDb': 0., 'thresh': -20, 'rise': 1.5, 'minSegLen': 0.05, 'maxSegLen': 1000}
		checkErrandKeywordArgs(self._defaults, kwargs)
		for k, v in self._defaults.iteritems(): setattr(self, k, kwargs.get(k, v))
	####
	def __repr__(self): return getClassPrintable('tsf', self, self.name)


class csf:
	def __init__(self, name, **kwargs):
		self.name = name
		self._defaults = {'Wnotehead': '.', 'hasParams': False, 'midiPitchMethod': 'composite', 'excludeStr': None, 'wholeFile': False, 'end': None, 'recursive': True, 'segmentationExtension': '.txt', 'includeStr': None, 'MWinstrName': '', 'start': None, 'onsetLen': 0.01, 'limit': {}, 'restrictInTime': 0, 'scaleDistance': 1, 'scaleDb': 0., 'MWtext': '', 'excludeTimes': [], 'limitDur': None, 'superimposeRule': None, 'offsetLen': '30%', 'envelopeSlope': 1, 'restrictOverlaps': None, 'concatFileName': None, 'segmentationFile': None, 'includeTimes': [], 'transMethod': None, 'transQuantize': 0, 'allowRepetition': True, 'restrictRepetition': 0.1}
		checkErrandKeywordArgs(self._defaults, kwargs)
		for k, v in self._defaults.iteritems():	
			setattr(self, k, kwargs.get(k, v))
	####
	def __repr__(self): return getClassPrintable('csf', self, self.name)


# search phase pass
class spass:
	def __init__(self, *args, **kwargs):	
		self._all_args = args
		self.method = args[0]
		self.descriptor_list = args[1:]
		self._defaults = {'percent': None, 'minratio': None, 'maxratio': None, 'complete_results': False}
		checkErrandKeywordArgs(self._defaults, kwargs)
		for k, v in self._defaults.iteritems(): setattr(self, k, kwargs.get(k, v))
		#####
		if self.method == 'closest': self.complete_results = True
		self.seg = []
		self.timevary = []
	####
	def __repr__(self): return getClassPrintable('spass', self, self._all_args)


# descriptor in spass	
class d:
	def __init__(self, descriptor_name, **kwargs):
		self.name = descriptor_name
		self._defaults = {'weight': 1, 'norm': 2, 'normmethod': 'stddev', 'distance': 'euclidean', 'energyWeight': False}
		checkErrandKeywordArgs(self._defaults, kwargs)
		for k, v in self._defaults.iteritems(): setattr(self, k, kwargs.get(k, v))
	####
	def __repr__(self): return getClassPrintable('d', self, self.name)
	




class si:
	def __init__(self, **kwargs):
		self._defaults = {'minSegment': None, 'maxSegment': None, 'minOnset': None, 'maxOnset': None, 'minOverlap': None, 'maxOverlap': None, 'overlapAmpThresh': -70., 'searchOrder': 'power', 'subtractScale': 1., 'subtractDur': 'corpusDur', 'ampDur': 'corpusDur', 'overlapDur': 'corpusEffDur', 'calcMethod': 'mixture', 'simCalcDur': 'corpusEffDur', 'peakAlign': False, 'incr': 1}
		for k, v in self._defaults.iteritems(): setattr(self, k, kwargs.get(k, v))
	####
	def __repr__(self): return getClassPrintable('si', self, None)
	####
	def test(self, name, val):
		if getattr(self, name) == None: return True



def getClassPrintable(name, obj, args):
	'''I hope I never have to understand this function'''
	spass = False
	if type(args) in [list, tuple]: # it is a "spass" object
		tmp = ['"%s"'%args[0]]
		for a in args[1:]: tmp.append(str(a))
		args = ', '.join(tmp)
		spass = True
	else: args = str(args)
	if spass:
		stry = ['%s'%args]
	elif args == 'None':
		stry = []
	else: 
		stry = ['"%s"'%args]
	for k, v in obj._defaults.iteritems():
		if getattr(obj, k) != v:
			if type(getattr(obj, k)) == str: thingy = '"%s"'%getattr(obj, k)
			else: thingy = str(getattr(obj, k))
			stry.append('%s=%s'%(k, thingy))
	return "%s(%s)"%(name, ', '.join(stry))
	


def file2optionsDict(file):
	# loads file into a dict
	dict = {}
	fh = open(file, 'r')
	lines = fh.read()
	fh.close()
	try:
		exec(lines, globals(), dict)
	except:
		utilities.error("ops", "there is a python syntax error in your options file\n\n\t"+str(sys.exc_info()[1]))
	return dict

def string2optionsDict(string):
	# loads file into a dict
	dict = {}
	try:
		exec(string, globals(), dict)
	except:
		utilities.error("ops", "there is a python syntax error in your options string\n\n\t"+str(sys.exc_info()[1]))
	return dict


# CHECK USER VARS AND MAKE SURE THAT THEY ARE THE CORRECT TYPE
opsCast = {
}

def checkOptionCasting(opsDict):
	for key, val in opsDict.iteritems():
		if opsCast.has_key(key):
			for castKey, castVal in opsCast[key].iteritems():
				if castKey == "type":
					if type(val) not in castVal:
						utilities.error('ops', 'option '+str(key)+' must be among the datatypes ->'+str(castVal))
				elif castKey == "len":
					if len(val) != castVal:
						utilities.error('ops', 'option '+str(key)+' must be '+str(castVal)+' items long')
				elif castKey == "among":
					if val not in castVal:
						utilities.error('ops', 'option '+str(key)+' must be one of the following choices '+str(castVal))
				elif castKey == "between":
					if type(val) not in [int, float] or (val < castVal[0] or val > castVal[1]):
						utilities.error('ops', 'option '+str(key)+' must be a number between '+str(castVal[0])+' and '+str(castVal[1]))

