#!/usr/bin/env python2.7
import sys, os, platform
import numpy as np
try:
	import json as json
except ImportError:
	try:
		import simplejson as json
	except: pass

ag_dir = sys.argv[1]
ops_file = sys.argv[2]
output_file_checksum = sys.argv[3]



def clusterAnalysis(clusteringMatrix, numb_clusters):# cluster analysis		
	Y = hcluster.pdist(clusteringMatrix)
	Z = hcluster.linkage(Y, method='average')
	clusterAssignment = hcluster.fcluster(Z, numb_clusters, criterion='maxclust') 
	clusterAssignment -= 1
	# get cluster centroids
	# Sum the vectors in each cluster
	lens = {}      # will contain the lengths for each cluster
	centroids = {} # will contain the centroids of each cluster
	for idx, clno in enumerate(clusterAssignment):
		centroids.setdefault(clno, np.zeros(clusteringMatrix.shape[1])) 
		centroids[clno] += clusteringMatrix[idx,:]
		lens.setdefault(clno,0)
		lens[clno] += 1
	# Divide by number of observations in each cluster to get the centroid
	for clno in centroids:
		centroids[clno] /= float(lens[clno])
	return clusterAssignment, centroids, lens



class BreakIt(Exception): pass
## ^ used for double-loop exiting
def mapClusterNodes(targetCentroids, corpusCentroids, numberClusters):
	#################
	#  BY DISTANCE	#
	#################
	minimumFoundError = (sys.maxint, None) # errorAmount, permSeriesIdx
	permutations = audioguide.utilities.permutations(range(numberClusters))

	calculatedDistances = {} # calculate pair-wise array distance only when not already done!

	for idx, perm in enumerate(permutations):
		thisError = 0.
		for t, c in enumerate(perm):
			try:
				if not calculatedDistances.has_key((t, c)):
					calculatedDistances[(t, c)] = np.sum(np.power(targetCentroids[t]-corpusCentroids[c], 2))
					#print t, c, calculatedDistances[(t, c)]
				thisError += calculatedDistances[(t, c)]
				if thisError > minimumFoundError[0]: raise BreakIt
			except BreakIt: pass
		if thisError < minimumFoundError[0]:
			minimumFoundError = (thisError, perm)
			#print "NEW MIN", minimumFoundError
		#print thisError
		#print idx

	tgtToCpsMap = {}
	for i in range(numberClusters):
		tgtToCpsMap[minimumFoundError[1][i]] = i
	return tgtToCpsMap


'''f.add_frame_type('1NEW', '1ABC NewMatrix, 1FQ0 New1FQ0')
f.add_matrix_type('1ABC', 'Column1, Column2')
# now you can begin adding frames
frame = f.new_frame('1NEW', time_now)
frame.add_matrix('1ABC', array([
    [0,     1.2],
    [3.5,   8.13],
    ...
    ]))
frame.write()'''



def sdif_testprint(sdfifile):
	sdfh = pysdif.SdifFile(sdfifile)
	print sdfifile
	for f in sdfh:
		print f.time, f.signature
		for m in f:
			print '\t', m.signature, '->', m.get_data(copy=False)
	# close this sdif file handle
	sdfh.close()


def writeSdifDictFiles(sdifDict):
	for sffile, dicty in sdifDict.iteritems():
		sdiffile = "%s-cluster-%s.sdif"%(SdifInterface.rawData[sffile]['sdiffileroot'], output_file_checksum)
		sdif = pysdif.SdifFile(sdiffile, 'w')
		for framename in dicty:
#			matrixid_list = []
#			for midx, matrixname in enumerate(dicty[framename].keys()):
#				matrixid_list.append('%s blah%i'%(matrixname, midx)) 
#			print "SDIF FRAME TYPE -- '%s %s'"%(framename, ', '.join(matrixid_list))
			for matrixname, matrix in dicty[framename].iteritems():
				m_str = []
				for i in range(matrix.shape[1]): m_str.append('Column%i'%(i+1))
				print "SDIF FRAME TYPE -- ", matrixname, matrixname+' blahdescript'
				print "SDIF MATRIX TYPE -- ", matrixname, ', '.join(m_str)
				sdif.add_frame_type(matrixname, matrixname+' blahdescript')
				sdif.add_matrix_type( matrixname, ', '.join(m_str) )
			#for matrixname, matrix in dicty[framename].iteritems():
				for f in xrange(matrix.shape[0]):
					frame = sdif.new_frame( matrixname, f*frameSec )
					frame.add_matrix( matrixname, matrix[f] )
					frame.write()
			# now you can begin adding frames
			sdif.close()
			print "\t\tWROTE", os.path.split(sdiffile)[1]
			sdif_testprint(sffile)
	#sys.exit(1)



# import audioguide submodules
sys.path.append( os.path.join(ag_dir, '..') ) # audioguide import
import audioguide
sys.path.append( audioguide.config['py_compiled_lib_dir'] )
import pysdif
import hcluster

main = audioguide.main(ag_dir, 'defaults.py', loadOptionsFromFile=ops_file)
main.loadTarget()
main.loadDatabase()
main.normalizeData()


#  just for convenience...
descriptorData = audioguide.descriptorData
tgt = audioguide.tgt
cps_handles = audioguide.ag.sourceHandles
descriptorData = audioguide.descriptorData
SdifInterface = audioguide.SdifInterface
frameSec = audioguide.ops.DESCRIPTOR_HOP_SIZE_SEC
numb_clusters = audioguide.ops.CLUSTERANAL_NUMB_CLUSTS

sdifDict = {}
descriptors = []
weights = []
for d in descriptorData.target_load:
	if d.weight == 0: continue
	descriptors.append(d)
	weights.append( d.weight )


########################
## do them separately ##
########################
###############
## do target ##
###############
descriptor_total_length = 0
for st, ed in tgt.segFrames: descriptor_total_length += ed-st
tgtDescriptorMatrix = np.empty((descriptor_total_length, len(descriptors)))
f_cnt = 0
for st, ed in tgt.segFrames:
	for f in xrange(ed-st):
		for didx, d in enumerate(descriptors):
			tgtDescriptorMatrix[f_cnt+f][didx] = tgt.data['norm'][d.name][st+f]
	f_cnt += ed-st

clusteringMatrix = tgtDescriptorMatrix.copy()*np.array(weights)
tcAssignment, targetCentroids, targetlens = clusterAnalysis(clusteringMatrix, numb_clusters)
print "TARGET:", targetlens


###############
## do corpus ##
###############
descriptor_total_length = 0
for h in cps_handles: descriptor_total_length += h.segdata.getraw('effDur-seg', 0, None)

# make emptry matrix
cpsDescriptorMatrix = np.empty((descriptor_total_length, len(descriptors)))
# fill it!
f_cnt = 0
for h in cps_handles:
	for f in xrange(h.segdata.getraw('effDur-seg', 0, None)):
		for didx, d in enumerate(descriptors):
			cpsDescriptorMatrix[f_cnt+f][didx] = h.data['norm'][d.name][f]
			#print h.name, h.startSec, d.name, clusteringMatrix[f_cnt+f][didx]
	f_cnt += h.segdata.getraw('effDur-seg', 0, None)

print "\t\tClustering data into %i clusters..."%numb_clusters
clusteringMatrix = cpsDescriptorMatrix.copy()*np.array(weights)
ccAssignment, corpusCentroids, corpuslens = clusterAnalysis(clusteringMatrix, numb_clusters)
print "CORPUS:", corpuslens

print "\t\tComputing least sqrs error cluster node mapping..."
tgtToCpsMap = mapClusterNodes(targetCentroids, corpusCentroids, numb_clusters)
# remap node ids
tmp = corpusCentroids.copy()
tmplens = corpuslens.copy()
corpuslens = {}
corpusCentroids = {}
for c, t in tgtToCpsMap.iteritems():
	corpusCentroids[t] = tmp[c]
	corpuslens[t] = tmplens[c]
for idx in range(len(ccAssignment)):
	ccAssignment[idx] = tgtToCpsMap[ccAssignment[idx]]

print ccAssignment


f_cnt = 0
sdifDict[tgt.filename] = {}
sdifDict[tgt.filename]['1CLT'] = {'1CSC':  np.zeros((tgt.lengthInFrames, len(descriptors))), '1CSD': np.zeros((tgt.lengthInFrames, 3))}
for st, ed in tgt.segFrames:
	for f in xrange(ed-st):
		for didx, d in enumerate(descriptors):
			sdifDict[tgt.filename]['1CLT']['1CSD'][st+f][0] = tcAssignment[f_cnt+f]
			sdifDict[tgt.filename]['1CLT']['1CSD'][st+f][1] = targetlens[tcAssignment[f_cnt+f]]
			diff = np.sum(np.power(tgtDescriptorMatrix[f_cnt+f]-targetCentroids[tcAssignment[f_cnt+f]], 2.))
			sdifDict[tgt.filename]['1CLT']['1CSD'][st+f][2] = diff

			sdifDict[tgt.filename]['1CLT']['1CSC'][st+f] = targetCentroids[tcAssignment[f_cnt+f]]
	f_cnt += ed-st


f_cnt = 0
for h in cps_handles:
	if not sdifDict.has_key(h.filename):
		lengthFrames = int(h.sfdur/frameSec)
		sdifDict[h.filename] = {}
		sdifDict[h.filename]['1CLT'] = {'1CSC':  np.zeros((lengthFrames, len(descriptors))), '1CSD': np.zeros((lengthFrames, 3))}
	st = int(h.startSec/frameSec)
	for f in xrange(h.segdata.getraw('effDur-seg', 0, None)):
		sdifDict[h.filename]['1CLT']['1CSD'][st+f][0] = ccAssignment[f_cnt+f] + 1
		sdifDict[h.filename]['1CLT']['1CSD'][st+f][1] = corpuslens[ccAssignment[f_cnt+f]]
		diff = np.sum(np.power(cpsDescriptorMatrix[f_cnt+f]-corpusCentroids[ccAssignment[f_cnt+f]], 2.))
		sdifDict[h.filename]['1CLT']['1CSD'][st+f][2] = diff	
		sdifDict[h.filename]['1CLT']['1CSC'][st+f] = corpusCentroids[ccAssignment[f_cnt+f]]
	f_cnt += h.segdata.getraw('effDur-seg', 0, None)
# now loop through files and create an sdif for each




writeSdifDictFiles(sdifDict)







#
#
#
#
#
#
#sys.exit(0)
#
#
## find length of all time varying metrics
#descriptor_total_length = 0
#for st, ed in tgt.segFrames: descriptor_total_length += ed-st
#for h in cps_handles: descriptor_total_length += h.segdata.getraw('effDur-seg', 0, None)
#
#
## make emptry matrix
#clusteringMatrix = np.empty((descriptor_total_length, len(descriptors)))
## fill it!
#f_cnt = 0
#for st, ed in tgt.segFrames:
#	for f in xrange(ed-st):
#		for didx, d in enumerate(descriptors):
#			clusteringMatrix[f_cnt+f][didx] = tgt.data['norm'][d.name][st+f]
#			#print d.name, clusteringMatrix[f_cnt+f][didx]
#	f_cnt += ed-st
#for h in cps_handles:
#	for f in xrange(h.segdata.getraw('effDur-seg', 0, None)):
#		for didx, d in enumerate(descriptors):
#			clusteringMatrix[f_cnt+f][didx] = h.data['norm'][d.name][f]
#			#print h.name, h.startSec, d.name, clusteringMatrix[f_cnt+f][didx]
#	f_cnt += h.segdata.getraw('effDur-seg', 0, None)
#
#
#clusteringMatrix *= np.array(weights)
#
#clusterAssignment, centroids, lens = clusterAnalysis(clusteringMatrix, numb_clusters)
#
#
## start counting from 1, 0 = no cluster found!
##		# put cluster assignments into a dict with file name and time as key.
#clusterDict = {}
#lengthFrames = int(tgt.sfdur/frameSec)
#clusterDict[tgt.filename] = np.zeros(lengthFrames)
#
#
#
#f_cnt = 0
#for st, ed in tgt.segFrames:
#	for f in xrange(ed-st):
#		clusterDict[tgt.filename][st+f] = clusterAssignment[f_cnt+f]
#	print clusterAssignment[f_cnt:f_cnt+(ed-st)]
#	f_cnt += ed-st
#for h in cps_handles:
#	if not clusterDict.has_key(h.filename):
#		lengthFrames = int(h.sfdur/frameSec)
#		clusterDict[h.filename] = np.zeros(lengthFrames)
#	startframe = int(h.startSec/frameSec)
#	for f in xrange(h.segdata.getraw('effDur-seg', 0, None)):
#		clusterDict[h.filename][startframe+f] = clusterAssignment[f_cnt+f]
#	f_cnt += h.segdata.getraw('effDur-seg', 0, None)
## now loop through files and create an sdif for each
#for sffile in clusterDict:
#	sdiffile = "%s-cluster-%s.sdif"%(SdifInterface.rawData[sffile]['sdiffileroot'], output_file_checksum)
#	# write a SdifFile
#	sdif = pysdif.SdifFile(sdiffile, 'w')
#	# define new frame and matrix types
#	sdif.add_frame_type('1CST', '1CST clusterid')
#	sdif.add_matrix_type('1CST', 'Column1')
#	# now you can begin adding frames
#	for f in xrange(len(clusterDict[sffile])):
#		frame = sdif.new_frame('1CST', f*frameSec)
#		frame.add_matrix('1CST', clusterDict[sffile][f:f+1] )
#		frame.write()
#	sdif.close()
#	print "WROTE", sdiffile	
#sys.exit(0)
#
#
#
#
#
#
