import sys, os
import numpy as np
sys.path.append('/Users/ben/Documents/audioGuide/audioguide/descriptorAnalysis/')
import descriptors
sys.path.append('/Users/ben/Documents/audioGuide/audioguide/')
import utilities as util
import userinterface
import scipy.io


#####################################
class matFileCollection:
	def __init__(self, filePath, matPaths, desList, requestedDescriptors):
		self.files = {}
		self.writeFlag = False
		for pidx, path in enumerate(matPaths):
			self.files[desList[pidx]] = {'arrayRow': pidx, 'matDict': {}, 'matpath': path, 'loadedMatFile': False}
		# load files of requested descriptors if found
		for descript in requestedDescriptors:
			if not os.path.exists(self.files[descript]['matpath']): continue # not here
			self.files[descript]['matDict'] = scipy.io.loadmat(self.files[descript]['matpath'])
#			self.files[descript]['hashList'] = self.files[descript]['matDict'].keys()
			self.files[descript]['loadedMatFile'] = True
	#################################
	def testSegHash(self, hash, requestedDescriptors):
		if hash in self.files[requestedDescriptors[0]]['matDict']:
			return True
		return False
	#################################
	def runAnalAndStore(self, sfUtil, segmentHash, filePath, startSec, endSec, scaleDb, onsetTime, offsetTime):
		MdArray = sfUtil.runAnalMultiDArray(filePath+" "+str(startSec)+" "+str(endSec)+" "+str(onsetTime)+" "+str(offsetTime))
		for fileKey, fileDict in self.files.iteritems():
			fileDict['matDict'][segmentHash] = MdArray[fileDict['arrayRow']][:-1]
			fileDict['matDict'][segmentHash+'-seg'] = MdArray[fileDict['arrayRow']][-1]
		self.writeFlag = True # write this sucker
	#################################
	def writeFiles(self):
		if not self.writeFlag: return # don't do anything...
		for descript, matFileDict in self.files.iteritems():
			# first we must test to see, if writing, that the data has been loaded!
			if os.path.exists(self.files[descript]['matpath']) and not self.files[descript]['loadedMatFile']:
				loaded = scipy.io.loadmat(self.files[descript]['matpath'])
				self.files[descript]['matDict'].update(loaded)
			print "writing", len(matFileDict['matDict'].keys()), "hashes to", os.path.split(matFileDict['matpath'])[1]
			scipy.io.savemat(matFileDict['matpath'], matFileDict['matDict'])
		self.files = {} # clear it just in case
###########################################################################




#####################################
class MatDataResources:
	def __init__(self):
		self.matCollObjs = {}
		self.sfUtil = descriptors.descriptorAnalysis(2048, 2, flags="-o n")
		self.availbleDescriptors = self.sfUtil.DesList[:] # hold all descriptors that can be given here
		for d in self.sfUtil.DesList: self.availbleDescriptors.append(d+'-seg')
	#################################
	def getmatPaths(self, sfPath):
		root, head = os.path.split(sfPath)
		paths = []
		for descriptor in self.sfUtil.DesList:
			paths.append(os.path.join(root, 'xml-descriptors', head+'-'+descriptor+'.mat'))
		return paths
	#################################
	def writeAllMatDataFiles(self):
		for filePath, matCollObj in self.matCollObjs.iteritems(): matCollObj.writeFiles()
	#################################
	# audioGuide accessable methods #
	#################################
	def retreiveNeededResources(self, requiredDescriptorList, requiredSegmentList, printerObj): # should be handles, not raw data!
		for (filePath, startSec, endSec, scaleDb, onsetTime, offsetTime) in requiredSegmentList:
			printerObj.barUp('1', os.path.split(filePath)[1]+"@"+str(util.trunc(startSec, 2)), inc=5)
			data = self.getDescriptors(requiredDescriptorList, filePath, startSec, endSec, scaleDb, onsetTime, offsetTime)
			#print data
	#################################
	def getDescriptors(self, descriptorList, filePath, startSec, endSec, scaleDb, onsetTime, offsetTime):
		descriptorsFound = []
		descriptorsNotFound = []
		for d in descriptorList:
			if d in self.availbleDescriptors: descriptorsFound.append(d)
			else: descriptorsNotFound.append(d)
		
		segmentHash = util.listToCheckSum([filePath, startSec, endSec, scaleDb, onsetTime, offsetTime])
		# create a new mat collection obj if none exists yet...
		if not self.matCollObjs.has_key(filePath):
			self.matCollObjs[filePath] = matFileCollection(filePath, self.getmatPaths(filePath), self.sfUtil.DesList, descriptorsFound)
		if not self.matCollObjs[filePath].testSegHash(segmentHash, descriptorList): # this segment is not there... add it
			self.matCollObjs[filePath].runAnalAndStore(self.sfUtil, segmentHash, filePath, startSec, endSec, scaleDb, onsetTime, offsetTime)
		# now return the requested descriptors in an audioguide dict.
		returnData = {}
		for descript in descriptorsFound:
			returnData[descript] = self.matCollObjs[filePath].files[descript]['matDict'][segmentHash]
			returnData[descript+'-seg'] = self.matCollObjs[filePath].files[descript]['matDict'][segmentHash+'-seg']
		return returnData, descriptorsNotFound
		
p = userinterface.Printer(3, True, 100)
matRc = MatDataResources()

reqDescript = ['mfcc0', 'mfcc1', 'mfcc2', 'mfcc3', 'mfcc4', 'mfcc5', 'mfcc6', 'mfcc7', 'power']

segs = []
audSegs = util.readAudacityLabelFile('/Users/ben/Documents/audioGuide/source/snareCatalogue.aiff.txt', False, False)

for seg in audSegs:
	segs.append(('/Users/ben/Documents/audioGuide/source/snareCatalogue.aiff', seg[0], seg[1], 0, 0.01, 0.05))

p.barOpen("Evaluating Corpus...", len(audSegs))
p.post("Parsing Corpus Segments", 3)
matRc.retreiveNeededResources(reqDescript, segs, p)
p.barClose("Read "+str(len(audSegs)))
matRc.writeAllMatDataFiles()